#define PC      1        fuer PC BORLAND C++ oder TURBO C++


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>                            /* getvect()... */
#define BIN_READ    "rb"

#define VERSION  "V1.00"                    /* Versionsnummer des Moduls */

typedef unsigned int uint;                  /* nuetzliche typedefs */
typedef unsigned char uchar;

/*********** RX wird gebuffert ********************************************/
#define RX_BUF_SIZE     512             /* Buffergroesse immer Potenz von 2 */
static uint ser_rinx=0;                 /* Index RX-Serielle Eingabe */
static uint ser_routx=0;                /* Index RX-Serielle Ausgabe */
static uchar ser_rx_buf[RX_BUF_SIZE];   /* Buffergroesse RX */

/*************************************************************************
*
* Serieller Handler PC
*
* Info zur Registerbelegung der PC-SIO z.B. in:  C t 5/88 S. 204
*
* Datum:    10. 12. 1992
* Compiler: Borland C++ 3.1
*
* Empfangen wird im Interrupt, gesendet via Direktzugriff auf SIO
*
*************************************************************************/

#define IRQ_CTRL        0x20            /* Interrupt-Controller */
#define IRQ_MASKE       0x21            /* Port fuer Interrupt-Maske */
#define SIO_EIN         0xE7            /* Sio-Bits im IRQ-Contr. */
#define EOI             0x20            /* Ende Interrupt */

#define COM1            0x3F8           /* Basisadr. COM1 */
#define COM2            0x2F8           /* Basisadr. COM2 */
                                        /* relative Offsets Register: */
#define IER             1               /* Interrupt Enable Reg. */
#define IIR             2               /* Interrupt Ident. Reg. */
#define LCR             3               /* Line Control Reg */
#define MCR             4               /* Modem Control */
#define LSR             5               /* Leitungs Status Reg */
#define MSR             6               /* Modem Status Reg */

#define COM1_VEC        0xC             /* Vektor SIO1 */
#define COM2_VEC        0xB             /* dto SIO2 */
#define TIMER_VEC       0x1C            /* Timer Vector */

static int com=COM1;                    /* Basisadresse */
static int com_vec=COM1_VEC;            /* Interrupt-Adresse */

static uint no_sio_mask;                /* Alte Interrupt-Maske */
static void interrupt (*o_com_vec)();   /* Alter SIO-Vektor */
static void interrupt (*o_timer_vec)(); /* Alter TIMER-Vector */

#define TIMEOUT 10                      /* Muss schnell sein! */
static volatile int klick_cnt=-1;       /* Klick-Zaaehler */

int hw_test(void);                      /* Borland C: Prototypen */
int reset(void);
                 
/*********** Timer-Interrupt **********************************************/
static void interrupt timer(){          /* Systemtimer zaehlt */
		if(klick_cnt>=0) klick_cnt--;       /* 18 mal pro Sekunde decr. */
}

/*********** Transfer-Interrupt: Daten Input-Daten buffern **************/
static void interrupt transfer(){

		ser_rx_buf[ser_rinx++]=inportb(com);
		ser_rinx &= (RX_BUF_SIZE-1);        /* Zyklischer Umlauf im RX-Buffer */
		klick_cnt=TIMEOUT;                  /* 0.25 sec. Stillhalten */
																				/* 32: Signalisiert TX-Buffer leer */
		if(inportb(com+LSR) & (2+4+8+16)){  /**** Fehler aufgetreten ? ****/
				inportb(com);
				inportb(com);
				inportb(com);
				inportb(com);
		}
		outportb(IRQ_CTRL,EOI);             /* Interrupt bearbeitet anzeigen! */
}
/******************* seriell_put_byte() *************************/
void envia_rs232c(register uint obyte){
		while(!(inportb(com+LSR) & 32));    /* Warten bis altes Byte draussen! */
		outportb(com,obyte);                /* 1 Byte raushauen */
}

/******* Senden ***********************************************************/
void senden(int typ, uchar* source, uint len){
		uchar tx_fcs=0;                 /* Pruefsumme Sender */
		int i;

		disable();                          /* Vor Senden RX-Buffer leeren */
		ser_rinx=ser_routx;
		enable();
		for(i=0;i<4;i++) inportb(com);      /* SIO leerraeumen */

    while(!(inportb(com+LSR) & 32));    /* Warten bis altes Byte draussen! */
    outportb(com,typ);                  /* 1. Byte ohne fcs raushauen */

    while(len--){
        while(!(inportb(com+LSR) & 32)); /* Warten bis altes Byte draussen! */
        outportb(com,*source);          /**** Byte absenden ****/
	tx_fcs += *source++;            /* Pruefsumme dazuzaehlen */
    }
    while(!(inportb(com+LSR) & 32)); /* Warten bis altes Byte draussen! */
    outportb(com,tx_fcs);               /* Pruefsumme ausgeben */
}

/******* Lesen ************************************************************/

/******** Ist was im Buffer ? *********************************************/
int seriell_rx_status(void){
    register uint delta;
    disable();
    delta=ser_rinx-ser_routx;
    enable();
    return delta?1:0;                        /* Nix da = 0, Was da = 1*/
}

/********* Ein Byte lesen *************************************************/
int recibe_rs232c(void){
    uchar ibyte;

    if(!seriell_rx_status()){                   /* nix vorhanden! */
        klick_cnt=TIMEOUT;
        do{
	    if(klick_cnt<0) return -1;          /* -1 Timeout */
        }while(!seriell_rx_status());           /* Solang nix da ist */
    }
    ibyte=ser_rx_buf[ser_routx++];              /* Byte holen */
    ser_routx &= (RX_BUF_SIZE-1);               /* Buffer Modulo rechnen */
    return ibyte;                               /* Gelesener Wert zurueck */
}

/******* lesen(): Zentrale Leseroutine: Angabe: wiviel Bytes und wohin
* Lesen von uint typ, len Bytes nach uchar. Letztes Byte ist fcs.
* fcs wird nur gelesen wenn len>0, FCS gilt nur fuer Datenblock */

int lesen(int typ, uchar* dest, uint len){
    uchar rx_fcs=0;             /* Pruefsumme lesen */
    register int inp;

    /* Zuerst Typ identifizieren */
    if((inp=recibe_rs232c())<0) return -1;     /* -1: Timeout - Typ */
    if(inp!=typ) return -2;                       /* -2: Typ - Fehler */

    while(len--){
        if((inp=recibe_rs232c())<0) return -3; /* -3: Timeout - Daten */
        rx_fcs+=(uchar)inp;                   /* FCS dazu-rechnen */
        *dest++=(uchar)inp;                   /* gelesenes Byte merken */
    }
    if((inp=recibe_rs232c())<0) return -4; /* -4: Timeout FCS */
    if(rx_fcs!=(uchar)inp) return -5;         /* -5: Fehler FCS */
    return 0;                                     /* Ansonst: OK! */
}

/******* InitCom() Initialisieren: Portnummer 1,2, Baudrate ****************/
void InitCom(int com_nr,long baudrate){
    uint divisor;
    int i;

    if(com_nr<1 || com_nr>2){
	puts("\007*** Error: Nmero del puerto serie incorrecto");
	exit(-1);
    }
    if(baudrate>115200L || baudrate<1L){
	printf("\007*** Error: No es posible seleccionar %ld baudios!\n",baudrate);
	exit(-1);
    }

    com=(com_nr-1)?COM2:COM1;           /* Basis-Adresse setzen */
    com_vec=(com_nr-1)?COM2_VEC:COM1_VEC;
    divisor=(uint)(115200L/baudrate);

    no_sio_mask=inportb(IRQ_MASKE) | (~SIO_EIN);  /* Kein SIO! */
    outportb(IRQ_MASKE,no_sio_mask);

    outportb(com+MCR,0);                /* Register loeschen */
    outportb(com+MSR,0);

    o_com_vec=getvect(com_vec);                  /* Alten Vektor holen */
    setvect(com_vec,transfer);                   /* Durch neuen ersetzen */
    o_timer_vec=getvect(TIMER_VEC);
    setvect(TIMER_VEC,timer);

    outportb(com+LCR,0x80);             /* Divisor-Latch-Enable Bit setzen */
    outportb(com,divisor&255);          /* Divosor Low */
    outportb(com+1,divisor>>8);         /* Divisor High */
    outportb(com+LCR,3);                /* 8 Bits, 1 Stopbit, keine Parity */

    outportb(com+IER,1);                /* Nur Empfangs-Irq */
    outportb(com+MCR,0xB);              /* Sio IRQ-Enable, RTS=DTR=1 */
    enable();                           /* Interrupts nun AN! */
    for(i=0;i<20;i++) inportb(com);     /* Muell entfernen */
    outportb(IRQ_MASKE, no_sio_mask & SIO_EIN); /* Daraus neue Maske */
}

/******** ExitCom: Aufraeumen *********************************************/
void termina_rs232c(void){                     /* Handler entfernen */
		setvect(com_vec,o_com_vec);         /* Alter Handler */
    setvect(TIMER_VEC,o_timer_vec);     /* Alter Timer */
    outportb(IRQ_MASKE,no_sio_mask);        /* Alte Maske */
}

/***************************************************************************
* Allgemeine Schnittstellenroutinen
****************************************************************************/

/******** reset: versuche Board zu resetten ********************************/
int reset(){

		/* Verhindere Falschmeldungen: Buffer leeren */
    while(seriell_rx_status()) recibe_rs232c();  /* Buffer leeren */

    /* '255' stoppt das momentan laufende Programm und korrigiert evtl. die
    * Baudrate auf 31250 Baud (MIDI) oder 9600 Baud (RS232) */
    envia_rs232c(255);                  /*** Stop! ***/
    if(recibe_rs232c()<0){               /* Byte 0..255 oder -1: Timeout */
	puts("\007*** Error: El Microcontrolador no responde!");
	return -1;
    }

    /* Moeglicherweise wurde Board gerade beim Senden unterbrochen. Daher
		* Buffer nochmal zu leeren versuchen */
    while(seriell_rx_status()) recibe_rs232c();  /* Buffer leeren */

    /* '135' fuehrt einen Warmstart auf dem Controlerboard durch.
    * Warmstart kann in seltenen Faellen (kreuzweise Interrupts,
    * Watchdog-Fehler etc... versagen */
    envia_rs232c(135);                  /**** Warmstart ****/
    if(recibe_rs232c()!=135){
	/* Wichtig: Warmstart immer pruefen! */
	puts("\007*** Error: No pudo arrancar en caliente!");
	return -1;
    }

    /* Board hat bis hierher korrekt geantwortet */
    return 0;
}

/****************** hw_test ************************************/
#define BLOCK    2048       /* Soviele Blockgroesse fuer Uebertragung,
			    * muss Potenz von 2 sein! */

int hw_test(){
    /* das Feld 'soll' wird gesendet. Dazu muss es im Header die Datenlaenge
    * und eine Zieladresse enthalten. Daher ist es um 4 Bytes groesser als
		* das Feld 'ist', das die empfangenen Werte aufnimmt */

    long adr=0x8000L;               /* Ab hier RAM */
    long len=32768L;                /* Soviel Bytes hat das Standard-RAM */

    static uchar ist[BLOCK], h_soll[BLOCK+4];
    uchar *soll;
    int i,in,lstat;

    soll=h_soll+4;                  /* Kopfbereich ueberspringen */

    while(len>0){
	printf("Probando-RAM: Direcciones $%04lx - $%04lx",
	    adr,adr+(long)(BLOCK-1));
	fflush(stdout);                 /* Manche Compiler: Zeile ausgeben */

	for(i=0;i<BLOCK;soll[i++]=i);   /* Testdaten zusammenbasteln */

	h_soll[0]=(uchar)(adr>>8);      /* Zieladresse high */
	h_soll[1]=(uchar)(adr&255);     /* Zieladresse low */
	h_soll[2]=(uchar)(BLOCK>>8);    /* Blocklaenge high */
	h_soll[3]=(uchar)(BLOCK&255);   /* Blocklaenge low */

	/* Datenblock ohne Rueckfrage an Board senden */
	senden(131,h_soll,BLOCK+4);     /* die 4 fuer den Header */
	/* Was antwortet MC darauf? */
	if((in=recibe_rs232c())<0){
	    puts(", Estado: \007*** Error: Ninguna respuesta!");
	    return -1;
	}

	if(in!=131){                    /* Protokoll verletzt! */
	    puts(", Estado: \007*** Se perdieron datos!");
	    return -1;
	}
	/* Adresse und Laenge des Datenkopfes bleiben, nur Typ wechselt */
	senden(130,h_soll,4);
	lstat=lesen(130,ist,BLOCK);
	if(lstat){           /* Fehlercodes von 'lesen' siehe dort */
	    puts(", Estado: \007*** Se perdieron datos!");
	    return -1;
	}
	/* Pruefen: SOLL und IST identisch? */
	for(i=0;i<BLOCK;i++){
	    if(ist[i]!=soll[i]){
		puts(", Estado: \007*** Error: No reconoce RAM!");
		return -1;
	    }
	}
	/* Dieser Block war in Ordnung! */
	puts(", Estado: OK - Banco de memoria correcto");

	/* Naechster Block auf neuer Adresse */
	len -= BLOCK;
	adr += BLOCK;
    }

    /* Alles hat geklappt, Board ist OK! */
    return 0;
}



void configura_rs232c(void)
{
	InitCom(1,9600);
}

void vacia_rs232c(void)
{
	while(seriell_rx_status()) {
		recibe_rs232c();
	}
}
